/*******************************************************************************
*    I²C Slave Library 1.0.                                                    *
*    Copyright (C) 2010-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#ifndef I2CSLAVE_H_
#define I2CSLAVE_H_

#define I2CSLAVE_VERSION    "1.0 Build 20100705"

// Wie viele Daten werden akzeptiert?
#define MAXRECEIVEDDATA     10
// Wie viele Daten können geschickt werden?
#define MAXSENDDATA         10
// Wie viele Register betreiben wir?
#define REGISTERCOUNT       2

#define SIG_DUNNO           0
#define SIG_START           1
#define SIG_REPSTART        2
#define SIG_STOP            3

#define NOREGISTERCHOOSED   0xFF
#define NODATA              0xFF

#define DUNNO               0
#define TRANSMIT            1
#define RECEIVE             2

#ifdef SOURCE_i2cslave
/**
* Millisekunden vom Einschalten/Reset
*/
volatile uint8_t registerAddress, dataCompleted;
volatile uint8_t receivedData[REGISTERCOUNT + 1][MAXRECEIVEDDATA], receivedDataCount[REGISTERCOUNT + 1];
volatile uint8_t toSendData[REGISTERCOUNT + 1][MAXSENDDATA], maxSendData[REGISTERCOUNT + 1];
#else
extern volatile uint8_t registerAddress, dataCompleted;
extern volatile uint8_t receivedData[REGISTERCOUNT + 1][MAXRECEIVEDDATA], receivedDataCount[REGISTERCOUNT + 1];
extern volatile uint8_t toSendData[REGISTERCOUNT + 1][MAXSENDDATA], maxSendData[REGISTERCOUNT + 1];
#endif

/**
 * Initialisierung des I²C-Slave-Subsystems
 *
 * @param uint8_t addr      I²C-Adresse des Gerätes
 */
void initI2Cslave(uint8_t adrr);

/**
 * Gibt zurück, ob Daten verfügbar sind
 *
 * @return                  TRUE wenn Daten verfügbar
 */
#define areDataAvailable()  (dataCompleted == 1)

/**
 * Nachdem die Daten empfangen worden sind, müssen sie
 * ungültig markiert werden, so daß sie NICHT mehrmals
 * bearbeitet werden.
 */
#define gotData()           {dataCompleted = 0;}

#endif /* I2CSLAVE_H_ */
