/*******************************************************************************
*    I²C Slave Library 1.0.                                                    *
*    Copyright (C) 2010-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <avr/wdt.h>

#ifdef SOURCE_tools
/**
* Millisekunden vom Einschalten/Reset
*/
volatile long uptime;
#else
extern volatile long uptime;
#endif

#define bzero(b,len) (memset((b), '\0', (len)), (void) 0)

inline void _delayFourCycles(unsigned int __count);

/**
* Initialisiert den Timer
*/
void doInitTimer(void);

/**
* Gibt den Uptime (Millisekunden seit dem letzten Einschalten/Reset)
* zurück
*
* @return long   Die Millisekunden seit dem letzten Einschalten/Reset
*/
#define getUptime()   uptime

/**
* Warte x Mikrosekunden
*
* @param uint8_t x   Die Mikrosekunden zu warten
*/
#define delayUS(__us)         _delayFourCycles( ( ( 1 * (F_CPU / 4000) ) * __us) / 1000 )

/**
* Warte x Millisekunden
*
* @param uint8_t x   Die Millisekunden zu warten
*/
void delayMS(unsigned int ms);

/**
* Warte x Sekunden
*
* @param uint8_t x   Die Sekunden zu warten
*/
#define delayS(__s)           { int __ls; for(__ls = 0; __ls < __s; __ls++) delayMS(1000); }

/**
* Rundet die angegebene Zahl
*
* @param uint8_t x   Die zu rundenden Zahl
*/
double round(double x);

/**
 * Warte bis zu einem bestimmten TimeOut, daß der eingegebenen PIN gesetzt wird.
 * Wenn es trotzdem "clear" bleibt, macht ein Reset
 *
 * @param __reg       Der Register
 * @param __bit       Der PIN
 * @param __timeOut   Der TimeOut in Millisekunden
 */
#define waitForPINSetWithTimeOut(__reg, __bit, __timeOut) \
{ \
  long start = getUptime(); \
  do \
  { \
    if(getUptime() > (start + __timeOut)) \
      resetMicro(); \
  } while (bit_is_clear(__reg, __bit)); \
}

/**
 * Warte bis zu einem bestimmten TimeOut, daß der eingegebenen PIN gelöscht wird.
 * Wenn es trotzdem "set" bleibt, macht ein Reset
 *
 * @param __reg       Der Register
 * @param __bit       Der PIN
 * @param __timeOut   Der TimeOut in Millisekunden
 */
#define waitForPINClearWithTimeOut(__reg, __bit, __timeOut) \
{ \
  long start = getUptime(); \
  do \
  { \
    if(getUptime() > (start + __timeOut)) \
      resetMicro(); \
  } while (bit_is_set(__reg, __bit)); \
}
