/*******************************************************************************
*    I²C Slave Library 1.0.                                                    *
*    Copyright (C) 2010-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <math.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "inc/constants.h"
#include "inc/leds.h"
#include "inc/tools.h"

volatile long ms8;

/*************************************************************************
 delay loop for small accurate delays: 16-bit counter, 4 cycles/loop
*************************************************************************/
inline void _delayFourCycles(unsigned int __count)
{
  if ( __count == 0 )
    __asm__ __volatile__( "rjmp 1f\n 1:" );       // 2 cycles
  else
    __asm__ __volatile__ (
                          "1: sbiw %0,1" "\n\t"
                          "brne 1b"               // 4 cycles/loop
                          : "=w" (__count)
                          : "0" (__count)
    );
}

/**
* Die Interrupt Service Routine (ISR).
* Die Variabel "uptime" wird erhöht
*/
SIGNAL (SIG_OUTPUT_COMPARE2)
{
  ms8++;
  if(ms8 == 8)
  {
    uptime++;
    ms8 = 0;
  }
}

/**
* Initialisiert den Timer
*/
void doInitTimer()
{
  uptime = 0L;
  ms8 = 0;

  // Timer2: keine PWM
  TCCR2 = 0;

  // Timer2 ist Zähler: Clear Timer on Compare Match (CTC, Mode #4)
  // Timer2 läuft mit vollem MCU-Takt: Prescale = 1
  TCCR2 = (1 << WGM21) | (0 << WGM20) | (0 << COM21) | (1 << COM20) | (1 << CS21);

  // OutputCompare für gewünschte Timer1 Frequenz
  // TCNT2 zählt immer 0...OCR2, 0...OCR2, ...
  // Beim überlauf OCR2 -> OCR2+1 wird TCNT2=0 gesetzt und im nächsten
  // MCU-Takt eine IRQ erzeugt.
  OCR2 = 200;

  // OutputCompare-Interrupt für Timer2 aktivieren
  TIMSK  |= (1 << OCIE2);
}

/**
* Warte x Millisekunden
*
* @param uint8_t x   Die Millisekunden zu warten
*/
void delayMS(unsigned int ms)
{
  int l;

  for(l = 0; l < ms; l++)
    delayUS(1000);
}

/**
* Rundet die angegebene Zahl
*
* @param uint8_t x   Die zu rundenden Zahl
*/
double round(double x)
{
  double intPart;
  double decPart;

  intPart = floor(x);
  decPart = x - intPart;
  if(decPart > 0.5)
    return (intPart + 1);
  else
    return intPart;
}
